@@ -9,11 +9,6 @@
 #include <string.h>
 #include <signal.h>
 #include <math.h>
-#ifdef MAEMO2
-  #include <hildon/hildon-file-chooser-dialog.h>
-  #include <hildon/hildon-defines.h>
-  #include <gdk/gdkkeysyms.h>
-#endif
 
 #include "callbacks.h"
 #include "interface.h"
@@ -49,53 +44,69 @@ void set_checkbox(void* main, gchar* name, gboolean val) {
 
 void set_main_title(gchar* session, gchar* extra) {
 	gchar *stmp;
-#ifdef MAEMO2
-	stmp = g_strconcat(session, " ", extra, NULL);
-#else
 	stmp = g_strconcat("Grsync: ", session, " ", extra, NULL);
-#endif
 	gtk_window_set_title((GtkWindow*) main_window, stmp);
 	g_free(stmp);
 }
 
 
-void load_settings(GtkWidget* main, gchar* session) {
+void load_settings(GtkWidget* main, gchar* session, gchar *filename) {
 	gchar settings_file_path[MAXPATH];
 	GError *error_handler = NULL;
 	GKeyFile *settings_file;
 	gchar *stmp;
 	gboolean btmp;
+	gint root_x, root_y, width, height;
+
+	if (filename == NULL) g_sprintf(settings_file_path, "%s/%s", grsync_dir, "grsync.ini");
+	else strncpy(settings_file_path, filename, MAXPATH-1);
 
-	g_sprintf(settings_file_path, "%s/%s", grsync_dir, "grsync.ini");
 	// initializes config_command before other functions which may use it
-	strcpy(config_command, "rsync");
+	if (filename == NULL) strcpy(config_command, "rsync");
 
 	if (!g_file_test(settings_file_path, G_FILE_TEST_EXISTS)) {
 		g_printf (_("(ERROR) Can't open config file! Maybe this is the first run?\n"));
-		save_settings(main, session);
-		load_groups(main, session);
+		if (session != NULL) {
+			save_settings(main, session, NULL);
+			load_groups(main, session);
+		}
 	}
 
-	set_main_title(session, NULL);
 	settings_file = g_key_file_new();
-	g_key_file_load_from_file(settings_file, settings_file_path, 	G_KEY_FILE_NONE, NULL);
+	g_key_file_load_from_file(settings_file, settings_file_path, G_KEY_FILE_NONE, NULL);
+	if (session == NULL) {
+		session = g_key_file_get_start_group(settings_file);
+		save_settings(main, session, NULL);
+		load_groups(main, session);
+	}
+	set_main_title(session, NULL);
 
-	// load configuration (preferences)
-	stmp = g_key_file_get_string(settings_file, CONFIG_GROUP, "command", NULL);
-	if (stmp != NULL) strncpy(config_command, stmp, MAXPATH - 1);
-	
-	btmp = g_key_file_get_boolean(settings_file, CONFIG_GROUP, "output", &error_handler);
-	if (error_handler != NULL) g_clear_error (&error_handler);
-	else config_output = btmp;
-	btmp = g_key_file_get_boolean(settings_file, CONFIG_GROUP, "remember", &error_handler);
-	if (error_handler != NULL) g_clear_error (&error_handler);
-	else config_remember = btmp;
-	btmp = g_key_file_get_boolean(settings_file, CONFIG_GROUP, "errorlist", &error_handler);
-	if (error_handler != NULL) g_clear_error (&error_handler);
-	else config_errorlist = btmp;
-	btmp = g_key_file_get_boolean(settings_file, CONFIG_GROUP, "logging", &error_handler);
-	if (error_handler != NULL) g_clear_error (&error_handler);
-	else config_log = btmp;
+	if (filename == NULL) {
+		// load configuration (preferences)
+		stmp = g_key_file_get_string(settings_file, CONFIG_GROUP, "command", NULL);
+		if (stmp != NULL) strncpy(config_command, stmp, MAXPATH - 1);
+		
+		btmp = g_key_file_get_boolean(settings_file, CONFIG_GROUP, "output", &error_handler);
+		if (error_handler != NULL) g_clear_error (&error_handler);
+		else config_output = btmp;
+		btmp = g_key_file_get_boolean(settings_file, CONFIG_GROUP, "remember", &error_handler);
+		if (error_handler != NULL) g_clear_error (&error_handler);
+		else config_remember = btmp;
+		btmp = g_key_file_get_boolean(settings_file, CONFIG_GROUP, "errorlist", &error_handler);
+		if (error_handler != NULL) g_clear_error (&error_handler);
+		else config_errorlist = btmp;
+		btmp = g_key_file_get_boolean(settings_file, CONFIG_GROUP, "logging", &error_handler);
+		if (error_handler != NULL) g_clear_error (&error_handler);
+		else config_log = btmp;
+
+		// load window position and size
+		root_x = g_key_file_get_integer(settings_file, CONFIG_GROUP, "root_x", NULL);
+		root_y = g_key_file_get_integer(settings_file, CONFIG_GROUP, "root_y", NULL);
+		width  = g_key_file_get_integer(settings_file, CONFIG_GROUP, "width", NULL);
+		height = g_key_file_get_integer(settings_file, CONFIG_GROUP, "height", NULL);
+		gtk_window_move((GtkWindow*) main_window, root_x, root_y);
+		if (width > 0 && height > 0) gtk_window_resize((GtkWindow*) main_window, width, height);
+	}
 
 	// load specific session
 	stmp = g_key_file_get_string(settings_file, session, "text_source", NULL);
@@ -129,6 +140,8 @@ void load_settings(GtkWidget* main, gchar* session) {
 	set_checkbox(main, "check_delete", g_key_file_get_boolean(settings_file, session, "check_delete", NULL));
 	set_checkbox(main, "check_exist", g_key_file_get_boolean(settings_file, session, "check_exist", NULL));
 	set_checkbox(main, "check_size", g_key_file_get_boolean(settings_file, session, "check_size", NULL));
+	set_checkbox(main, "check_skipnew", g_key_file_get_boolean(settings_file, session, "check_skipnew", NULL));
+	set_checkbox(main, "check_windows", g_key_file_get_boolean(settings_file, session, "check_windows", NULL));
 
 	set_checkbox(main, "check_sum", g_key_file_get_boolean(settings_file, session, "check_sum", NULL));
 	set_checkbox(main, "check_symlink", g_key_file_get_boolean(settings_file, session, "check_symlink", NULL));
@@ -147,28 +160,41 @@ void load_settings(GtkWidget* main, gchar* session) {
 }
 
 
-void save_settings(GtkWidget* main, gchar *session) {
+void save_settings(GtkWidget* main, gchar *session, gchar *filename) {
 	gchar settings_file_name[MAXPATH];
 	gchar *key_data;
 	FILE *key_file;
 	GError *error_handler = NULL;
 	GKeyFile *settings_file;
 	const gchar *stmp;
+	gint root_x, root_y, width, height;
 
-	g_sprintf(settings_file_name, "%s/%s", grsync_dir, "grsync.ini");
+	if (filename == NULL) g_sprintf(settings_file_name, "%s/%s", grsync_dir, "grsync.ini");
+	else strncpy(settings_file_name, filename, MAXPATH-1);
 
 //	g_printf(_("Saving settings to %s...\n"), settings_file_name);
 	g_mkdir(grsync_dir, 0700);
 	settings_file = g_key_file_new();
-	g_key_file_load_from_file(settings_file, settings_file_name, G_KEY_FILE_NONE, NULL);
+	
+	if (filename == NULL) {
+		g_key_file_load_from_file(settings_file, settings_file_name, G_KEY_FILE_NONE, NULL);
 
-	// save configuration (preferences)
-	g_key_file_set_string(settings_file, CONFIG_GROUP, "command", config_command);
-	g_key_file_set_boolean(settings_file, CONFIG_GROUP, "output", config_output);
-	g_key_file_set_boolean(settings_file, CONFIG_GROUP, "remember", config_remember);
-	g_key_file_set_boolean(settings_file, CONFIG_GROUP, "errorlist", config_errorlist);
-	g_key_file_set_boolean(settings_file, CONFIG_GROUP, "logging", config_log);
-	if (session_last != -1) g_key_file_set_string(settings_file, CONFIG_GROUP, "last_session", groups[session_last]);
+		// save configuration (preferences)
+		g_key_file_set_string(settings_file, CONFIG_GROUP, "command", config_command);
+		g_key_file_set_boolean(settings_file, CONFIG_GROUP, "output", config_output);
+		g_key_file_set_boolean(settings_file, CONFIG_GROUP, "remember", config_remember);
+		g_key_file_set_boolean(settings_file, CONFIG_GROUP, "errorlist", config_errorlist);
+		g_key_file_set_boolean(settings_file, CONFIG_GROUP, "logging", config_log);
+		if (session_last != -1) g_key_file_set_string(settings_file, CONFIG_GROUP, "last_session", groups[session_last]);
+
+		// window position and size
+		gtk_window_get_position((GtkWindow*) main_window, &root_x, &root_y);
+		gtk_window_get_size((GtkWindow*) main_window, &width, &height);
+		g_key_file_set_integer(settings_file, CONFIG_GROUP, "root_x", root_x);
+		g_key_file_set_integer(settings_file, CONFIG_GROUP, "root_y", root_y);
+		g_key_file_set_integer(settings_file, CONFIG_GROUP, "width", width);
+		g_key_file_set_integer(settings_file, CONFIG_GROUP, "height", height);
+	}
 
 	// save specific session
 	stmp = gtk_entry_get_text((GtkEntry*) lookup_widget((GtkWidget*) main, "text_source"));
@@ -192,6 +218,8 @@ void save_settings(GtkWidget* main, gchar *session) {
 	g_key_file_set_boolean(settings_file, session, "check_delete", get_checkbox(main, "check_delete"));
 	g_key_file_set_boolean(settings_file, session, "check_exist", get_checkbox(main, "check_exist"));
 	g_key_file_set_boolean(settings_file, session, "check_size", get_checkbox(main, "check_size"));
+	g_key_file_set_boolean(settings_file, session, "check_skipnew", get_checkbox(main, "check_skipnew"));
+	g_key_file_set_boolean(settings_file, session, "check_windows", get_checkbox(main, "check_windows"));
 
 	g_key_file_set_boolean(settings_file, session, "check_sum", get_checkbox(main, "check_sum"));
 	g_key_file_set_boolean(settings_file, session, "check_symlink", get_checkbox(main, "check_symlink"));
@@ -299,13 +327,9 @@ void show_browse_source(GtkButton *button, gpointer user_data) {
 	GtkFileChooserAction action = browse_files ? GTK_FILE_CHOOSER_ACTION_OPEN : GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
 	gchar *curr_path = (gchar*) gtk_entry_get_text((GtkEntry*) lookup_widget((GtkWidget*) button, "text_source"));
 	gchar *curr_folder = g_path_get_dirname(curr_path);
-
-#ifdef MAEMO2
-	dialog = hildon_file_chooser_dialog_new (NULL, action);
-#else
+		
 	dialog = gtk_file_chooser_dialog_new (_("Browse"), (GtkWindow*) main_window, action,
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
-#endif
 	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), curr_folder);
 	if (browse_files) gtk_file_chooser_select_filename(GTK_FILE_CHOOSER (dialog), curr_path);
 
@@ -330,13 +354,9 @@ void show_browse_dest(GtkButton *button, gpointer user_data) {
 	GtkFileChooserAction action = browse_files ? GTK_FILE_CHOOSER_ACTION_OPEN : GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
 	gchar *curr_path = (gchar*) gtk_entry_get_text((GtkEntry*) lookup_widget((GtkWidget*) button, "text_dest"));
 	gchar *curr_folder = g_path_get_dirname(curr_path);
-
-#ifdef MAEMO2
-	dialog = hildon_file_chooser_dialog_new (NULL, action);
-#else
+		
 	dialog = gtk_file_chooser_dialog_new (_("Browse"), (GtkWindow*) main_window, action,
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
-#endif
 	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), curr_folder);
 	if (browse_files) gtk_file_chooser_select_filename(GTK_FILE_CHOOSER (dialog), curr_path);
 
@@ -374,6 +394,8 @@ void on_play_clicked(GtkButton *button, gpointer user_data) {
 	if (get_checkbox(button, "check_delete")) argv[i++] = "--delete";
 	if (get_checkbox(button, "check_exist")) argv[i++] = "--ignore-existing";
 	if (get_checkbox(button, "check_size")) argv[i++] = "--size-only";
+	if (get_checkbox(button, "check_skipnew")) argv[i++] = "-u";
+	if (get_checkbox(button, "check_windows")) argv[i++] = "--modify-window=1";
 
 	if (get_checkbox(button, "check_sum")) argv[i++] = "-c";
 	if (get_checkbox(button, "check_symlink")) argv[i++] = "-l";
@@ -467,7 +489,7 @@ void on_main_create(GtkWindow *window, gpointer user_data) {
 		argv_session = g_strconcat("default", NULL);
 		load_groups((GtkWidget*) window, argv_session);
 	}
-	load_settings((GtkWidget*) window, argv_session);
+	load_settings((GtkWidget*) window, argv_session, NULL);
 
 	stmp = g_strconcat(config_command, " --help", NULL);
 	if (!g_spawn_command_line_sync(stmp, &rsync_help, NULL, NULL, NULL)) {
@@ -476,8 +498,7 @@ void on_main_create(GtkWindow *window, gpointer user_data) {
 		gtk_dialog_run(GTK_DIALOG(error));
 		gtk_widget_destroy(error);
 	} else {
-#ifndef MAEMO2
-	//	g_spawn_command_line_sync("man rsync", &rsync_man, NULL, NULL, NULL);
+//		g_spawn_command_line_sync("man rsync", &rsync_man, NULL, NULL, NULL);
 		rsync_tips = gtk_tooltips_new();
 		set_tooltip(rsync_tips, "--times", "check_time");
 		set_tooltip(rsync_tips, "--perms", "check_perm");
@@ -489,6 +510,9 @@ void on_main_create(GtkWindow *window, gpointer user_data) {
 		set_tooltip(rsync_tips, "--delete", "check_delete");
 		set_tooltip(rsync_tips, "--ignore-existing", "check_exist");
 		set_tooltip(rsync_tips, "--size-only", "check_size");
+		set_tooltip(rsync_tips, "--update", "check_skipnew");
+		set_tooltip(rsync_tips, "--modify-window=NUM", "check_windows");
+
 		set_tooltip(rsync_tips, "--checksum", "check_sum");
 		set_tooltip(rsync_tips, "--links", "check_symlink");
 		set_tooltip(rsync_tips, "-D", "check_dev");
@@ -497,7 +521,6 @@ void on_main_create(GtkWindow *window, gpointer user_data) {
 		set_tooltip(rsync_tips, "--numeric-ids", "check_mapuser");
 		set_tooltip(rsync_tips, "--compress", "check_compr");
 		set_tooltip(rsync_tips, "--backup", "check_backup");
-#endif
 	}
 	g_free(stmp);
 	if (cmdline_execute) on_play_clicked((GtkButton*) window, NULL);
@@ -506,7 +529,7 @@ void on_main_create(GtkWindow *window, gpointer user_data) {
 
 void on_main_destroy(GtkObject *object, gpointer user_data) {
 	gint sel = gtk_combo_box_get_active((GtkComboBox*) lookup_widget((GtkWidget*) object, "combo_session"));
-	save_settings((GtkWidget*) object, groups[sel]);
+	save_settings((GtkWidget*) object, groups[sel], NULL);
 	g_strfreev(groups);
 	if (rsync_help != NULL) g_free(rsync_help);
 	if (rsync_man != NULL) g_free(rsync_man);
@@ -771,12 +794,6 @@ void on_rsync_show(GtkWidget *widget, gpointer user_data) {
 	g_free(comm_str);
 
 	view = (GtkTextView*)lookup_widget(widget, "textview_output");
-	#ifdef MAEMO2
-	PangoFontDescription *font_desc = pango_font_description_new ();
-	pango_font_description_set_size (font_desc, 10 * PANGO_SCALE);
-	gtk_widget_modify_font (GTK_WIDGET (view), font_desc);
-	pango_font_description_free (font_desc);
-	#endif
 	buffer = gtk_text_view_get_buffer(view);
 	gtk_text_buffer_insert_at_cursor(buffer, comm_text, -1);
 	if (config_log) fputs(comm_text, log_file);
@@ -821,9 +838,9 @@ void on_rsync_destroy(GtkObject *object, gpointer user_data) {
 void on_combo_session_changed (GtkComboBox *combobox, gpointer user_data) {
 	gint sel = gtk_combo_box_get_active((GtkComboBox*) lookup_widget((GtkWidget*) combobox, "combo_session"));
 	if (session_last != -1 && sel != session_last) {
-		save_settings((GtkWidget*) combobox, groups[session_last]);
+		save_settings((GtkWidget*) combobox, groups[session_last], NULL);
 		session_last = sel;
-		load_settings((GtkWidget*) combobox, groups[session_last]);
+		load_settings((GtkWidget*) combobox, groups[session_last], NULL);
 	}
 }
 
@@ -859,7 +876,7 @@ void on_session_add_clicked(GtkButton *button, gpointer user_data) {
 			return;
 		}
 
-		save_settings((GtkWidget*) button, newses);
+		save_settings((GtkWidget*) button, newses, NULL);
 		session_number++;
 		load_groups((GtkWidget*) button, newses);
 	}
@@ -909,7 +926,7 @@ void on_session_del_clicked(GtkButton *button, gpointer user_data) {
 		g_clear_error(&error_handler);
 		session_last = -1;
 		load_groups((GtkWidget*) button, "default");
-		load_settings((GtkWidget*) button, "default");
+		load_settings((GtkWidget*) button, "default", NULL);
 	}
 }
 
@@ -988,14 +1005,6 @@ void on_rsync_info_activate(GtkMenuItem *menuitem, gpointer user_data) {
 
 	tmps = g_strconcat(config_command, " --version", NULL);
 	g_spawn_command_line_sync(tmps, &result, NULL, NULL, NULL);
-#ifdef MAEMO2
-	if (result) {
-		gchar *nn;
-		nn = strstr(result, "\n\n");
-		if (nn) *nn = 0;
-		g_free(nn);
-	}
-#endif
 	dialog = gtk_message_dialog_new((GtkWindow*) main_window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE, result);
 	gtk_dialog_run(GTK_DIALOG(dialog));
 
@@ -1004,16 +1013,51 @@ void on_rsync_info_activate(GtkMenuItem *menuitem, gpointer user_data) {
 	g_free(result);
 }
 
-#ifdef MAEMO2
-gboolean hardware_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer user_data) {
-	switch (event->keyval) {
-		case HILDON_HARDKEY_FULLSCREEN :
-			if (!(gdk_window_get_state (event->window) &
-			    GDK_WINDOW_STATE_FULLSCREEN)) gtk_window_fullscreen (GTK_WINDOW (widget));
-			else gtk_window_unfullscreen (GTK_WINDOW (widget));
-			return TRUE;
-	}	
-	return FALSE;
-}
-#endif
 
+void on_import1_activate(GtkMenuItem *menuitem, gpointer user_data) {
+	GtkWidget *dialog, *dialog2;
+	gint retval, i;
+		
+	dialog = gtk_file_chooser_dialog_new (_("Browse"), (GtkWindow*) main_window, GTK_FILE_CHOOSER_ACTION_OPEN,
+		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
+	retval = gtk_dialog_run (GTK_DIALOG (dialog));
+	if (retval == GTK_RESPONSE_ACCEPT || retval == GTK_RESPONSE_OK) {
+		gchar *filename;
+		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+		load_settings((GtkWidget*) menuitem, NULL, filename);
+
+		dialog2 = gtk_message_dialog_new((GtkWindow*) main_window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
+			_("Session imported"));
+		gtk_dialog_run(GTK_DIALOG(dialog2));
+		gtk_widget_destroy(dialog2);
+		g_free(filename);
+	}
+	gtk_widget_destroy (dialog);
+}
+
+
+void on_export1_activate (GtkMenuItem *menuitem, gpointer user_data) {
+	GtkWidget *dialog, *dialog2;
+	gchar *filename;
+	gint retval, i;
+		
+	i = gtk_combo_box_get_active((GtkComboBox*) lookup_widget((GtkWidget*) menuitem, "combo_session"));
+	dialog = gtk_file_chooser_dialog_new (_("Browse"), (GtkWindow*) main_window, GTK_FILE_CHOOSER_ACTION_SAVE,
+		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
+	filename = g_strconcat(groups[i], ".grsync", NULL);
+	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), filename);
+	g_free(filename);
+
+	retval = gtk_dialog_run (GTK_DIALOG (dialog));
+	if (retval == GTK_RESPONSE_ACCEPT || retval == GTK_RESPONSE_OK) {
+		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+		save_settings((GtkWidget*) menuitem, groups[i], filename);
+
+		dialog2 = gtk_message_dialog_new((GtkWindow*) main_window, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
+			_("Session exported"));
+		gtk_dialog_run(GTK_DIALOG(dialog2));
+		gtk_widget_destroy(dialog2);
+		g_free(filename);
+	}
+	gtk_widget_destroy (dialog);
+}
