@@ -4,8 +4,8 @@
 #define MAXOPT 100
 #define CONFIG_GROUP "__CONFIG"
 
-void load_settings(GtkWidget* main, gchar *session);
-void save_settings(GtkWidget* main, gchar *session);
+void load_settings(GtkWidget* main, gchar *session, gchar *filename);
+void save_settings(GtkWidget* main, gchar *session, gchar *filename);
 gboolean load_groups(GtkWidget* main, gchar *session);
 
 void
@@ -126,7 +126,10 @@ void
 on_rsync_destroy                       (GtkObject       *object,
                                         gpointer         user_data);
 
-#ifdef MAEMO2
-gboolean hardware_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer user_data);
-#endif
+void
+on_import1_activate                    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
 
+void
+on_export1_activate                    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
